import { csv } from 'd3';
import {
  select,
  json,
  pie,
  arc,
  scaleOrdinal,
  format
} from 'd3';
import { ScaleOrdinal } from 'd3';
import { schemeCategory10 } from 'd3-scale-chromatic';

// Colors for the sections in Pie Chart
const colorScale = scaleOrdinal(schemeCategory10)
  .domain(Array.from({ length: 9 }, (_, i) => i));
const sectionColors = Array.from({ length: 9 }, (_, i) => colorScale(i));

// Data set URI
const csvUrl = 'https://gist.githubusercontent.com/anvesh2129/714f12672563f4d4011004403b07e861/raw/c1133d318a9e826c47c5903680c407ba4e9b9002/Game_sales.csv';

// Function to fetch data from the URI
const fetchData = async () => {
  const data = await csv(csvUrl);
  return data;
};

const svg = select("svg");
const width = + svg.attr('width');
const height = + svg.attr('height');

// Format axes text
const xAxisTickFormat = number =>
  	format('.4~g')(number);

// Render Pie Chart on the canvas
const render = (data, total) => {
  const pieData = pie().value(d=>d.Total_Sales)(data);

  // Scale to determine colors for sections based on Total Sales
  const colors = scaleOrdinal()
  	.domain(data)
  	.range(sectionColors);

  // Object for generating Arcs of the sections
  const arcGenerator = d3.arc()
  .innerRadius(0)
  .outerRadius(250);

  // Define properties for each segment in the Pie Chart
	const segments = arc()
  	.innerRadius(0)
  	.outerRadius(175)
  	.padAngle(0)
  	.padRadius(0);
  
  // Select all sections in the pie chart
  const sections = svg.append("g")
  	.attr("transform", `translate(250,200)`)
  	.selectAll("path").data(pieData);

  // Render each section
  sections.enter()
    .append("path")
    .attr("d", segments)
  	.attr("fill", d => colors(d.data.Total_Sales));

  // Render text on each section
  sections
    .enter()
    .append('text')
    .text(d => xAxisTickFormat((d.data.Total_Sales/total*100))+"%")
    .attr("transform", function(d) { return "translate(" + arcGenerator.centroid(d) + ")";  })
    .style("text-anchor", "middle")
  	.style('fill', 'black')
    .style("font-size", 10);

  // Render legend for the data
  const legends = svg.append("g")
  	.attr("transform", "translate(500,100)")
    .selectAll(".legends").data(pieData);

  // Render each legend item
  const legend = legends.enter().append("g").classed(".legends",true)
  	.attr("transform", (d,i)=>{
      return `translate(0,${(i+1)*26})`;
    });

  // Add color box for each legend item
  legend.append("rect").attr("width",20).attr("height",20)
  	.attr("fill", d => colors(d.data.Total_Sales));

// Add text element for the game name to the legend
legend.append("text")
  .attr("x", 35) // Set the x coordinate for the text element
  .attr("y", 15) // Set the y coordinate for the text element
  .attr("class","legend_text") // Add a class to the text element for styling purposes
  .text(d => d.data.Game_Name); // Set the text content for the text element to the name of the game
  
// Add text element for the percentage value to the legend
legend.append("text")
  .attr("x", 350) // Set the x coordinate for the text element
  .attr("y", 15) // Set the y coordinate for the text element
  .attr("class","legend_value") // Add a class to the text element for styling purposes
  .text(d=>xAxisTickFormat((d.data.Total_Sales/total*100))+"%"); // Set the text content for the text element to the percentage value of the game's total sales, formatted as a percentage
}

// Main function
const run = async () => {
  let data = await fetchData(); // Fetch data from the URI
  let total = 0;
  data.forEach(d=>{
    d.Total_Sales = +d.Total_Sales;
    total += d.Total_Sales;
  }); // Convert Total_Sales property to number and calculate the total sales across all games
  render(data,total); // Render the pie chart and legend using the data and total sales values
};
run(); // Call the run function to start the program